#!/usr/bin/perl
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted
# material is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Milan Zazrivec

use warnings;
use strict;

print "Running IPv6 network next hop reachibility test\n\n".
      "Test will scan IPv6 routing table for next hops and test\n".
      "their reachibility using ICMP ping6.\n\n";

my $line;
my ($flags, $interface, $gw, $next_hop);
my $p = qr/^.+\ .+\ .+\ .+\ (.+)\ .+\ .+\ .+\ (.+)\b\ +\b(.+)$/x;
my %gateways6;
my $address;
my $errors = 0;

# scan /proc/net/ipv6_route for gateways
die "$!\n" unless open FILE, "/proc/net/ipv6_route";
while ($line = <FILE>)
{
    chomp $line;
    next unless ($line =~ $p);
    $next_hop = $1;
    $flags = hex($2);
    $interface = $3;
    next unless (($flags & 2) == 2); # RTF_GATEWAY flag
    # unpack, pack trick -- we need $next_hop in standard IPv6 notation
    $next_hop =~ s/([0-9A-F]{32})/
                 join ":", unpack "H[4]H[4]H[4]H[4]H[4]H[4]H[4]H[4]",
                 pack "H*", $1/eig;
    $gateways6{"$next_hop"} = $interface;
}
close FILE;

if (%gateways6)
{
    print "Found following configured IPv6 'next hops' ".
          "(address/interface) pairs:\n";
    foreach $address (keys %gateways6)
    {
        print "$address / $gateways6{$address}\n";
    }
}
else
{
    print "Test did not find any hops in system's IPv6 routing table.\n".
          "Exiting test.\n";
    exit 0;
}

foreach $address (keys %gateways6)
{
    # we do not have any IPv6 capable arping utility, just ping6
    print "\nICMPv6 ping to $address... ";
    if (system("/bin/ping6 -c 3 -I $gateways6{ $address } $address " .
               " >& /dev/null") != 0)
    {
        print "Failed.\n".
              "Next hop $address (IPv6) could not be reached (ICMP)\n";
        ++$errors;
    }
    else
    {
        print "Succeeded.\n";
    }
}

if (not $errors)
{
    print "\nTest complete: result=PASS\n";
}
else
{
    print "\nTest complete: result=FAIL\n";
}

exit $errors;
